#include <MCUFRIEND_kbv.h>

#define MARGIN 10
#define INPUT_PIN A15
#define THRESHOLD 3

#define BLACK   0x0000
#define BLUE    0x001F
#define RED     0xF800
#define GREEN   0x07E0
#define CYAN    0x07FF
#define MAGENTA 0xF81F
#define YELLOW  0xFFE0
#define WHITE   0xFFFF

MCUFRIEND_kbv tft;
uint16_t colors[] = {RED, YELLOW, GREEN, CYAN, BLUE, MAGENTA};
int colorsLength = sizeof(colors) / sizeof(uint16_t);
int sliderWidth;
int colorWidth;
int lastValue = -1;

void setup() {
  tft.begin(tft.readID());
  tft.setRotation(1);
  sliderWidth = tft.width() - 2 * MARGIN;
  colorWidth = sliderWidth / colorsLength;
  initScreen();
}

void loop() {
  int value = analogRead(INPUT_PIN);
  if (value < lastValue - THRESHOLD || value > lastValue + THRESHOLD) {
    int cursor = map(value, 0, 1023, 0, sliderWidth);
    int colorIndex = selectColor(cursor);
    if (colorIndex != -1) {
      drawCursor(colors[colorIndex], cursor);
      printValue(colors[colorIndex], "INPUT", String(value));
    }
    lastValue = value;
  }
  delay(1);
}

void initScreen() {
  tft.fillScreen(BLACK);

  tft.setCursor(MARGIN, MARGIN);
  tft.print("Tutorial di Arduino");
  tft.setCursor(MARGIN, MARGIN+15);
  tft.setTextColor(YELLOW);
  tft.setTextSize(2);
  tft.print("Display TFT Touch");
  tft.drawLine(MARGIN, MARGIN + 40, tft.width() - MARGIN, MARGIN + 40, WHITE);

  tft.fillRect(MARGIN, 122, sliderWidth, 2, WHITE);
  for (int i = 0; i < colorsLength; i++) {
    tft.fillRect(MARGIN + i * colorWidth, 117, 2, 5, WHITE);
    tft.fillRect(MARGIN + i * colorWidth, 130, colorWidth, 30, colors[i]);
  }
  tft.fillRect(MARGIN + colorsLength * colorWidth - 2, 117, 2, 5, WHITE);
}

int selectColor(int cursor) {
  for (int i = 0; i < colorsLength; i++) {
    if (cursor >= i * colorWidth && cursor < (i + 1) * colorWidth) {
      return i;
    }
  }
  return -1;
}

void drawCursor(uint16_t color, int cursor) {
  int pos = MARGIN + cursor;
  tft.fillRect(0, 85, tft.width(), 26, BLACK);
  tft.fillRect(pos - 2, 85, 5, 15, color);
  tft.fillTriangle(pos - 9, 100, pos, 110, pos + 9, 100, color);
}

void printValue(uint16_t color, String label, String value) {
  tft.setTextColor(color);
  tft.setTextSize(3);
  tft.setCursor(MARGIN, tft.height() - MARGIN - 22);
  tft.print(label + ": ");
  tft.fillRect(MARGIN + 125, tft.height() - MARGIN - 23, tft.width(), 25, BLACK);
  tft.setCursor(MARGIN + 130, tft.height() - MARGIN - 22);
  tft.print(value);
}
